@@ -1,5 +1,8 @@
+import errno
+import inspect
 import os
 import typing
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -22,16 +25,55 @@ class Process:
         process_func: processing function,
             which expects the two positional arguments ``signal``
             and ``sampling_rate``
-            and any number of additional keyword arguments.
+            and any number of additional keyword arguments
+            (see ``process_func_args``).
+            There are the following special arguments:
+            ``'idx'``, ``'file'``, ``'root'``.
+            If expected by the function,
+            but not specified in
+            ``process_func_args``,
+            they will be replaced with:
+            a running index,
+            the currently processed file,
+            the root folder.
+            There is no restriction on the return type of the function
+        process_func_args: (keyword) arguments passed on to the processing
+            function
         process_func_is_mono: if set to ``True`` and the input signal
             has multiple channels, ``process_func`` will be applied to
             every channel individually
         sampling_rate: sampling rate in Hz.
             If ``None`` it will call ``process_func`` with the actual
-            sampling rate of the signal.
+            sampling rate of the signal
         resample: if ``True`` enforces given sampling rate by resampling
         channels: channel selection, see :func:`audresample.remix`
         mixdown: apply mono mix-down on selection
+        win_dur: window duration,
+            if processing should be applied on a sliding window.
+            If value is a float or integer
+            it is treated as seconds.
+            See :func:`audinterface.utils.to_timedelta` for further options
+        hop_dur: hop duration,
+            if processing should be applied on a sliding window.
+            This defines the shift between two windows.
+            If value is a float or integer
+            it is treated as seconds.
+            See :func:`audinterface.utils.to_timedelta` for further options.
+            Defaults to ``win_dur / 2``
+        min_signal_dur: minimum signal length
+            required by ``process_func``.
+            If value is a float or integer
+            it is treated as seconds.
+            See :func:`audinterface.utils.to_timedelta` for further options.
+            If provided signal is shorter,
+            it will be zero padded at the end
+        max_signal_dur: maximum signal length
+            required by ``process_func``.
+            If value is a float or integer
+            it is treated as seconds.
+            See :func:`audinterface.utils.to_timedelta` for further options.
+            If provided signal is longer,
+            it will be cut at the end
         segment: when a :class:`audinterface.Segment` object is provided,
             it will be used to find a segmentation of the input signal.
             Afterwards processing is applied to each segment
@@ -44,21 +86,61 @@ class Process:
             multiprocessing
         multiprocessing: use multiprocessing instead of multithreading
         verbose: show debug messages
-        kwargs: additional keyword arguments to the processing function
 
     Raises:
         ValueError: if ``resample = True``, but ``sampling_rate = None``
+        ValueError: if ``hop_dur`` is specified, but not ``win_dur``
 
-    """
+    Example:
+        >>> def mean(signal, sampling_rate):
+        ...     return signal.mean()
+        >>> interface = Process(process_func=mean)
+        >>> signal = np.array([1., 2., 3.])
+        >>> interface(signal, sampling_rate=3)
+        2.0
+        >>> interface.process_signal(signal, sampling_rate=3)
+        start   end
+        0 days  0 days 00:00:01   2.0
+        dtype: float64
+        >>> import audb
+        >>> db = audb.load(
+        ...     'emodb',
+        ...     version='1.2.0',
+        ...     media='wav/03a01Fa.wav',
+        ...     full_path=False,
+        ...     verbose=False,
+        ... )
+        >>> index = db['emotion'].index
+        >>> interface.process_index(index, root=db.root)
+        file             start   end
+        wav/03a01Fa.wav  0 days  0 days 00:00:01.898250    -0.000311
+        dtype: float32
+        >>> interface = Process(
+        ...     process_func=mean,
+        ...     win_dur=1.0,
+        ...     hop_dur=0.5,
+        ... )
+        >>> interface.process_index(index, root=db.root)
+        file             start                   end
+        wav/03a01Fa.wav  0 days 00:00:00         0 days 00:00:01          -0.000329
+                         0 days 00:00:00.500000  0 days 00:00:01.500000   -0.000285
+        dtype: float32
+
+    """  # noqa: E501
     def __init__(
             self,
             *,
             process_func: typing.Callable[..., typing.Any] = None,
+            process_func_args: typing.Dict[str, typing.Any] = None,
             process_func_is_mono: bool = False,
             sampling_rate: int = None,
             resample: bool = False,
             channels: typing.Union[int, typing.Sequence[int]] = None,
             mixdown: bool = False,
+            win_dur: Timestamp = None,
+            hop_dur: Timestamp = None,
+            min_signal_dur: Timestamp = None,
+            max_signal_dur: Timestamp = None,
             segment: Segment = None,
             keep_nat: bool = False,
             num_workers: typing.Optional[int] = 1,
@@ -66,47 +148,111 @@ class Process:
             verbose: bool = False,
             **kwargs,
     ):
+        if channels is not None:
+            channels = audeer.to_list(channels)
+
+        if process_func is None:
+            def process_func(signal, _):
+                return signal
+
+        process_func_args = process_func_args or {}
+        if kwargs:
+            warnings.warn(
+                utils.kwargs_deprecation_warning,
+                category=UserWarning,
+                stacklevel=2,
+            )
+            for key, value in kwargs.items():
+                process_func_args[key] = value
+
         if resample and sampling_rate is None:
             raise ValueError(
                 'sampling_rate has to be provided for resample = True.'
             )
-        self.sampling_rate = sampling_rate
-        r"""Sampling rate in Hz."""
-        self.resample = resample
-        r"""Resample signal."""
-        self.channels = None if channels is None else audeer.to_list(channels)
+
+        if win_dur is None and hop_dur is not None:
+            raise ValueError(
+                "You have to specify 'win_dur' if 'hop_dur' is given."
+            )
+        if win_dur is not None and hop_dur is None:
+            hop_dur = utils.to_timedelta(win_dur, sampling_rate) / 2
+
+        # figure out if special arguments
+        # to pass to the processing function
+        signature = inspect.signature(process_func)
+        self._process_func_special_args = {
+            'idx': False,
+            'root': False,
+            'file': False,
+        }
+        for key in self._process_func_special_args:
+            if (
+                    key in signature.parameters
+                    and key not in process_func_args
+            ):
+                self._process_func_special_args[key] = True
+
+        self.channels = channels
         r"""Channel selection."""
-        self.mixdown = mixdown
-        r"""Mono mixdown."""
-        self.segment = segment
-        r"""Segmentation object."""
+
         self.keep_nat = keep_nat
         r"""Keep NaT in results."""
-        self.num_workers = num_workers
-        r"""Number of workers."""
+
+        self.hop_dur = hop_dur
+        r"""Hop duration."""
+
+        self.max_signal_dur = max_signal_dur
+        r"""Maximum signal length."""
+
+        self.min_signal_dur = min_signal_dur
+        r"""Minimum signal length."""
+
+        self.mixdown = mixdown
+        r"""Mono mixdown."""
+
         self.multiprocessing = multiprocessing
         r"""Use multiprocessing."""
-        self.verbose = verbose
-        r"""Show debug messages."""
-        if process_func is None:
-            def process_func(signal, _):
-                return signal
+
+        self.num_workers = num_workers
+        r"""Number of workers."""
+
         self.process_func = process_func
         r"""Processing function."""
+
+        self.process_func_args = process_func_args
+        r"""Additional keyword arguments to processing function."""
+
         self.process_func_is_mono = process_func_is_mono
         r"""Process channels individually."""
-        self.process_func_kwargs = kwargs
-        r"""Additional keyword arguments to processing function."""
+
+        self.resample = resample
+        r"""Resample signal."""
+
+        self.sampling_rate = sampling_rate
+        r"""Sampling rate in Hz."""
+
+        self.segment = segment
+        r"""Segmentation object."""
+
+        self.verbose = verbose
+        r"""Show debug messages."""
+
+        self.win_dur = win_dur
+        r"""Window duration."""
 
     def _process_file(
             self,
             file: str,
             *,
+            idx: int = 0,
+            root: str = None,
             start: pd.Timedelta = None,
             end: pd.Timedelta = None,
-            root: str = None,
     ) -> pd.Series:
 
+        start = utils.to_timedelta(start, self.sampling_rate)
+        end = utils.to_timedelta(end, self.sampling_rate)
+
         signal, sampling_rate = utils.read_audio(
             file,
             start=start,
@@ -116,15 +262,32 @@ class Process:
         y = self._process_signal(
             signal,
             sampling_rate,
+            idx=idx,
+            root=root,
             file=file,
         )
 
-        if start is None or pd.isna(start):
-            start = y.index.levels[1][0]
-        if end is None or (pd.isna(end) and not self.keep_nat):
-            end = y.index.levels[2][0] + start
+        if self.win_dur is not None:
 
-        y.index = y.index.set_levels([[start], [end]], [1, 2])
+            if start is not None:
+                starts = y.index.levels[1] + start
+                ends = y.index.levels[2] + start
+                y.index = y.index.set_levels(
+                    [starts, ends],
+                    level=[1, 2],
+                )
+
+        else:
+
+            if start is None or pd.isna(start):
+                start = y.index.levels[1][0]
+            if end is None or (pd.isna(end) and not self.keep_nat):
+                end = y.index.levels[2][0] + start
+
+            y.index = y.index.set_levels(
+                [[start], [end]],
+                level=[1, 2],
+            )
 
         return y
 
@@ -141,9 +304,11 @@ class Process:
         Args:
             file: file path
             start: start processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
             end: end processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
             root: root folder to expand relative file path
 
         Returns:
@@ -156,8 +321,6 @@ class Process:
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
-        start = utils.to_timedelta(start)
-        end = utils.to_timedelta(end)
         if self.segment is not None:
             index = self.segment.process_file(
                 file,
@@ -167,7 +330,12 @@ class Process:
             )
             return self._process_index_wo_segment(index, root)
         else:
-            return self._process_file(file, start=start, end=end, root=root)
+            return self._process_file(
+                file,
+                root=root,
+                start=start,
+                end=end,
+            )
 
     def process_files(
             self,
@@ -183,9 +351,13 @@ class Process:
             files: list of file paths
             starts: segment start positions.
                 Time values given as float or integers are treated as seconds.
+                See :func:`audinterface.utils.to_timedelta`
+                for further options.
                 If a scalar is given, it is applied to all files
             ends: segment end positions.
-                Time values given as float or integers are treated as seconds
+                Time values given as float or integers are treated as seconds.
+                See :func:`audinterface.utils.to_timedelta`
+                for further options.
                 If a scalar is given, it is applied to all files
             root: root folder to expand relative file paths
 
@@ -199,28 +371,31 @@ class Process:
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
+        if len(files) == 0:
+            return pd.Series(dtype=object)
+
         if isinstance(starts, (type(None), float, int, str, pd.Timedelta)):
             starts = [starts] * len(files)
         if isinstance(ends, (type(None), float, int, str, pd.Timedelta)):
             ends = [ends] * len(files)
 
-        starts = utils.to_timedelta(starts)
-        ends = utils.to_timedelta(ends)
-
         params = [
             (
                 (file, ),
                 {
+                    'idx': idx,
+                    'root': root,
                     'start': start,
                     'end': end,
-                    'root': root,
                 },
-            ) for file, start, end in zip(files, starts, ends)
+            ) for idx, (file, start, end)
+            in enumerate(zip(files, starts, ends))
         ]
+
         verbose = self.verbose
         self.verbose = False  # avoid nested progress bar
         y = audeer.run_tasks(
-            self.process_file,
+            self._process_file,
             params,
             num_workers=self.num_workers,
             multiprocessing=self.multiprocessing,
@@ -248,12 +423,21 @@ class Process:
             Series with processed files conform to audformat_
 
         Raises:
+            FileNotFoundError: if folder does not exist
             RuntimeError: if sampling rates do not match
             RuntimeError: if channel selection is invalid
 
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
+        root = audeer.safe_path(root)
+        if not os.path.exists(root):
+            raise FileNotFoundError(
+                errno.ENOENT,
+                os.strerror(errno.ENOENT),
+                root,
+            )
+
         files = audeer.list_file_names(root, filetype=filetype)
         files = [os.path.join(root, os.path.basename(f)) for f in files]
         return self.process_files(files)
@@ -265,19 +449,21 @@ class Process:
     ) -> pd.Series:
         r"""Like process_index, but does not apply segmentation."""
         if index.empty:
-            return pd.Series(None, index=index, dtype=float)
+            return pd.Series(None, index=index, dtype=object)
 
         params = [
             (
                 (file, ),
                 {
+                    'idx': idx,
+                    'root': root,
                     'start': start,
                     'end': end,
-                    'root': root,
                 },
             )
-            for file, start, end in index
+            for idx, (file, start, end) in enumerate(index)
         ]
+
         y = audeer.run_tasks(
             self._process_file,
             params,
@@ -294,12 +480,22 @@ class Process:
             index: pd.Index,
             *,
             root: str = None,
+            cache_root: str = None,
     ) -> pd.Series:
         r"""Process from an index conform to audformat_.
 
+        If ``cache_root`` is not ``None``,
+        a hash value is created from the index
+        using :func:`audformat.utils.hash` and
+        the result is stored as
+        ``<cache_root>/<hash>.pkl``.
+        When called again with the same index,
+        features will be read from the cached file.
+
         Args:
             index: index with segment information
             root: root folder to expand relative file paths
+            cache_root: cache folder (see description)
 
         Returns:
             Series with processed segments conform to audformat_
@@ -311,18 +507,35 @@ class Process:
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
-        index = audformat.utils.to_segmented_index(index)
+        cache_path = None
 
-        if self.segment is not None:
-            index = self.segment.process_index(index, root=root)
+        if cache_root is not None:
+            cache_root = audeer.mkdir(cache_root)
+            hash = audformat.utils.hash(index)
+            cache_path = os.path.join(cache_root, f'{hash}.pkl')
 
-        return self._process_index_wo_segment(index, root)
+        if cache_path and os.path.exists(cache_path):
+            y = pd.read_pickle(cache_path)
+        else:
+            index = audformat.utils.to_segmented_index(index)
+
+            if self.segment is not None:
+                index = self.segment.process_index(index, root=root)
+
+            y = self._process_index_wo_segment(index, root)
+
+            if cache_path is not None:
+                y.to_pickle(cache_path, protocol=4)
+
+        return y
 
     def _process_signal(
             self,
             signal: np.ndarray,
             sampling_rate: int,
             *,
+            idx: int = 0,
+            root: str = None,
             file: str = None,
             start: pd.Timedelta = None,
             end: pd.Timedelta = None,
@@ -339,18 +552,69 @@ class Process:
             signal, sampling_rate, start, end,
         )
 
-        # Trim and process signal
-        y = self(signal[:, start_i:end_i], sampling_rate)
+        # Trim signal and ensure it has requested min/max length
+        signal = signal[:, start_i:end_i]
+        num_samples = signal.shape[1]
+        if self.max_signal_dur is not None:
+            max_signal_dur = utils.to_timedelta(
+                self.max_signal_dur,
+                sampling_rate,
+            )
+            max_samples = int(
+                max_signal_dur.total_seconds()
+                * sampling_rate
+            )
+            if num_samples > max_samples:
+                end = start + max_signal_dur
+                signal = signal[:, :max_samples]
+        if self.min_signal_dur is not None:
+            min_signal_dur = utils.to_timedelta(
+                self.min_signal_dur,
+                sampling_rate,
+            )
+            min_samples = int(
+                min_signal_dur.total_seconds()
+                * sampling_rate
+            )
+            if num_samples < min_samples:
+                end = start + min_signal_dur
+                num_pad = min_samples - num_samples
+                signal = np.pad(signal, ((0, 0), (0, num_pad)), 'constant')
+
+        # Process signal
+        y = self._call(
+            signal,
+            sampling_rate,
+            idx=idx,
+            root=root,
+            file=file,
+        )
 
         # Create index
-        if file is not None:
-            index = audformat.segmented_index(file, start, end)
-        else:
-            index = pd.MultiIndex.from_tuples(
-                [(start, end)], names=['start', 'end']
+        if self.win_dur is not None:
+            win_dur = utils.to_timedelta(self.win_dur, sampling_rate)
+            hop_dur = utils.to_timedelta(self.hop_dur, sampling_rate)
+            starts = pd.timedelta_range(
+                start,
+                freq=hop_dur,
+                periods=len(y),
             )
+            ends = starts + win_dur
+        else:
+            starts = [start]
+            ends = [end]
+            y = [y]
 
-        return pd.Series([y], index)
+        if file is not None:
+            files = [file] * len(starts)
+            index = audformat.segmented_index(files, starts, ends)
+        else:
+            index = utils.signal_index(starts, ends)
+
+        if len(y) == 0:
+            return pd.Series([], index, dtype=object)
+        else:
+            return pd.Series(y, index)
 
     def process_signal(
             self,
@@ -372,9 +636,11 @@ class Process:
             sampling_rate: sampling rate in Hz
             file: file path
             start: start processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
             end: end processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
 
         Returns:
             Series with processed signal conform to audformat_
@@ -386,8 +652,6 @@ class Process:
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
-        start = utils.to_timedelta(start)
-        end = utils.to_timedelta(end)
         if self.segment is not None:
             index = self.segment.process_signal(
                 signal,
@@ -402,6 +666,8 @@ class Process:
                 index,
             )
         else:
+            start = utils.to_timedelta(start, sampling_rate)
+            end = utils.to_timedelta(end, sampling_rate)
             return self._process_signal(
                 signal,
                 sampling_rate,
@@ -419,22 +685,31 @@ class Process:
         r"""Like process_signal_from_index, but does not apply segmentation."""
 
         if index.empty:
-            return pd.Series(None, index=index, dtype=float)
+            return pd.Series(None, index=index, dtype=object)
 
         if isinstance(index, pd.MultiIndex) and len(index.levels) == 2:
             params = [
                 (
                     (signal, sampling_rate),
-                    {'start': start, 'end': end},
-                ) for start, end in index
+                    {
+                        'idx': idx,
+                        'start': start,
+                        'end': end,
+                    },
+                ) for idx, (start, end) in enumerate(index)
             ]
         else:
             index = audformat.utils.to_segmented_index(index)
             params = [
                 (
                     (signal, sampling_rate),
-                    {'file': file, 'start': start, 'end': end},
-                ) for file, start, end in index
+                    {
+                        'idx': idx,
+                        'file': file,
+                        'start': start,
+                        'end': end,
+                    },
+                ) for idx, (file, start, end) in enumerate(index)
             ]
 
         y = audeer.run_tasks(
@@ -463,6 +738,7 @@ class Process:
                 or a :class:`pandas.MultiIndex` with two levels
                 named `start` and `end` that hold start and end
                 positions as :class:`pandas.Timedelta` objects.
+                See also :func:`audinterface.utils.signal_index`
 
         Returns:
             Series with processed segments conform to audformat_
@@ -470,14 +746,15 @@ class Process:
         Raises:
             RuntimeError: if sampling rates do not match
             RuntimeError: if channel selection is invalid
+            ValueError: if index contains duplicates
 
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
-        utils.check_index(index)
+        utils.assert_index(index)
 
         if index.empty:
-            return pd.Series(None, index=index, dtype=float)
+            return pd.Series(None, index=index, dtype=object)
 
         if self.segment is not None:
             index = self.segment.process_signal_from_index(
@@ -492,319 +769,71 @@ class Process:
             index,
         )
 
-    @audeer.deprecated(
-        removal_version='0.8.0',
-        alternative='process_index',
-    )
-    def process_unified_format_index(
-            self,
-            index: pd.Index,
-    ) -> pd.Series:  # pragma: nocover
-        r"""Process from an index conform to the `Unified Format`_.
-
-        .. note:: It is assumed that the index already holds segments,
-            i.e. in case a ``segment`` object is given, it will be ignored.
-
-        Args:
-            index: index with segment information
-
-        Returns:
-            Series with processed segments in the Unified Format
-
-        Raises:
-            RuntimeError: if sampling rates do not match
-            RuntimeError: if channel selection is invalid
-
-        .. _`Unified Format`: http://tools.pp.audeering.com/audata/
-            data-tables.html
-
-        """
-
-        index = audformat.utils.to_segmented_index(index)
-        utils.check_index(index)
-
-        if index.empty:
-            return pd.Series(None, index=index, dtype=float)
-
-        params = [
-            (
-                (file, ),
-                {
-                    'start': start,
-                    'end': end,
-                },
-            )
-            for file, start, end in index
-        ]
-        y = audeer.run_tasks(
-            self._process_file,
-            params,
-            num_workers=self.num_workers,
-            multiprocessing=self.multiprocessing,
-            progress_bar=self.verbose,
-            task_description=f'Process {len(index)} segments',
-        )
-
-        return pd.concat(y)
-
-    def __call__(
+    def _call(
             self,
             signal: np.ndarray,
             sampling_rate: int,
-    ) -> typing.Any:
-        r"""Apply processing to signal.
-
-        This function processes the signal **without** transforming the output
-        into a :class:`pd.Series`. Instead it will return the raw processed
-        signal. However, if channel selection, mixdown and/or resampling
-        is enabled, the signal will be first remixed and resampled if the
-        input sampling rate does not fit the expected sampling rate.
-
-        Args:
-            signal: signal values
-            sampling_rate: sampling rate in Hz
-
-        Returns:
-            Processed signal
-
-        Raises:
-            RuntimeError: if sampling rates do not match
-            RuntimeError: if channel selection is invalid
-
-        """
-        signal, sampling_rate = utils.preprocess_signal(
-            signal,
-            sampling_rate,
-            self.sampling_rate,
-            self.resample,
-            self.channels,
-            self.mixdown,
-        )
-        if self.process_func_is_mono and signal.shape[0] > 1:
-            return [
-                self.process_func(
-                    np.atleast_2d(channel),
-                    sampling_rate,
-                    **self.process_func_kwargs,
-                ) for channel in signal
-            ]
-        return self.process_func(
-            signal,
-            sampling_rate,
-            **self.process_func_kwargs,
-        )
-
-
-class ProcessWithContext:
-    r"""Alternate processing interface that provides signal context.
-
-    In contrast to :class:`Process` this interface does not look at segments
-    in isolation, but passes the complete signal together with a list of
-    segments to the processing function. By doing so, it becomes possible to
-    process segments in context, e.g. by taking into account surrounding
-    signal values or other segments.
-
-    Args:
-        process_func: processing function, which expects four positional
-            arguments:
-
-            * ``signal``
-            * ``sampling_rate``
-            * ``starts`` sequence with start indices
-            * ``ends`` sequence with end indices
-
-            and any number of additional keyword arguments.
-            Must return a sequence of results for every segment.
-        sampling_rate: sampling rate in Hz.
-            If ``None`` it will call ``process_func`` with the actual
-            sampling rate of the signal.
-        resample: if ``True`` enforces given sampling rate by resampling
-        channels: channel selection, see :func:`audresample.remix`
-        mixdown: apply mono mix-down on selection
-        verbose: show debug messages
-        kwargs: additional keyword arguments to the processing function
-
-    Raises:
-        ValueError: if ``resample = True``, but ``sampling_rate = None``
-
-    """
-    def __init__(
-            self,
-            *,
-            process_func: typing.Callable[
-                ...,
-                typing.Sequence[typing.Any]
-            ] = None,
-            sampling_rate: int = None,
-            resample: bool = False,
-            channels: typing.Union[int, typing.Sequence[int]] = None,
-            mixdown: bool = False,
-            verbose: bool = False,
-            **kwargs,
-    ):
-        if resample and sampling_rate is None:
-            raise ValueError(
-                'sampling_rate has to be provided for resample = True.'
-            )
-        self.sampling_rate = sampling_rate
-        r"""Sampling rate in Hz."""
-        self.resample = resample
-        r"""Resample signal."""
-        self.channels = None if channels is None else audeer.to_list(channels)
-        r"""Channel selection."""
-        self.mixdown = mixdown
-        r"""Mono mixdown."""
-        self.verbose = verbose
-        r"""Show debug messages."""
-        if process_func is None:
-            def process_func(signal, _, starts, ends):
-                return [
-                    signal[:, start:end] for start, end in zip(starts, ends)
-                ]
-        self.process_func = process_func
-        r"""Process function."""
-        self.process_func_kwargs = kwargs
-        r"""Additional keyword arguments to processing function."""
-
-    def process_index(
-            self,
-            index: pd.Index,
             *,
+            idx: int = 0,
             root: str = None,
-    ) -> pd.Series:
-        r"""Process from a segmented index conform to audformat_.
+            file: str = None,
+    ) -> typing.Any:
+        r"""Call processing function, possibly pass special args."""
 
-        Args:
-            index: index with segment information
-            root: root folder to expand relative file paths
+        signal, sampling_rate = utils.preprocess_signal(
+            signal,
+            sampling_rate,
+            self.sampling_rate,
+            self.resample,
+            self.channels,
+            self.mixdown,
+        )
 
-        Returns:
-            Series with processed segments conform to audformat_
+        special_args = {}
+        for key, value in [
+            ('idx', idx),
+            ('root', root),
+            ('file', file),
+        ]:
+            if self._process_func_special_args[key]:
+                special_args[key] = value
 
-        Raises:
-            RuntimeError: if sampling rates do not match
-            RuntimeError: if channel selection is invalid
-
-        .. _audformat: https://audeering.github.io/audformat/data-format.html
-
-        """
-        index = audformat.utils.to_segmented_index(index)
-
-        if index.empty:
-            return pd.Series(index=index, dtype=float)
-
-        files = index.levels[0]
-        ys = [None] * len(files)
-
-        with audeer.progress_bar(
-                files,
-                total=len(files),
-                disable=not self.verbose,
-        ) as pbar:
-            for idx, file in enumerate(pbar):
-                desc = audeer.format_display_message(file, pbar=True)
-                pbar.set_description(desc, refresh=True)
-                mask = index.isin([file], 0)
-                select = index[mask].droplevel(0)
-                signal, sampling_rate = utils.read_audio(file, root=root)
-                ys[idx] = pd.Series(
-                    self.process_signal_from_index(
-                        signal, sampling_rate, select,
-                    ).values,
-                    index=index[mask],
+        def _helper(x):
+            if self.process_func_is_mono:
+                return [
+                    self.process_func(
+                        np.atleast_2d(channel),
+                        sampling_rate,
+                        **special_args,
+                        **self.process_func_args,
+                    ) for channel in x
+                ]
+            else:
+                return self.process_func(
+                    x,
+                    sampling_rate,
+                    **special_args,
+                    **self.process_func_args,
                 )
 
-        return pd.concat(ys)
+        if self.win_dur is not None:
+            frames = utils.sliding_window(
+                signal,
+                sampling_rate,
+                self.win_dur,
+                self.hop_dur,
+            )
+            num_frames = frames.shape[-1]
+            y = [_helper(frames[..., idx]) for idx in range(num_frames)]
+        else:
+            y = _helper(signal)
 
-    def process_signal_from_index(
+        return y
+
+    def __call__(
             self,
             signal: np.ndarray,
             sampling_rate: int,
-            index: pd.Index,
-    ) -> pd.Series:
-        r"""Split a signal into segments and process each segment.
-
-        Args:
-            signal: signal values
-            sampling_rate: sampling rate in Hz
-            index: a :class:`pandas.MultiIndex` with two levels
-                named `start` and `end` that hold start and end
-                positions as :class:`pandas.Timedelta` objects.
-
-        Returns:
-            Series with processed segments conform to audformat_
-
-        Raises:
-            RuntimeError: if sampling rates do not match
-            RuntimeError: if channel selection is invalid
-
-        .. _audformat: https://audeering.github.io/audformat/data-format.html
-
-        """
-        utils.check_index(index)
-
-        starts_i, ends_i = utils.segments_to_indices(
-            signal, sampling_rate, index,
-        )
-        y = self(signal, sampling_rate, starts_i, ends_i)
-
-        return pd.Series(y, index=index)
-
-    @audeer.deprecated(
-        removal_version='0.8.0',
-        alternative='process_index',
-    )
-    def process_unified_format_index(
-            self,
-            index: pd.Index,
-    ) -> pd.Series:  # pragma: nocover
-        r"""Process from a index conform to the `Unified Format`_.
-
-        Args:
-            index: index with segment information
-
-        Returns:
-            Series with processed segments in the Unified Format
-
-        Raises:
-            RuntimeError: if sampling rates do not match
-            RuntimeError: if channel selection is invalid
-
-        """
-        index = audformat.utils.to_segmented_index(index)
-
-        if index.empty:
-            return pd.Series(index=index, dtype=float)
-
-        files = index.levels[0]
-        ys = [None] * len(files)
-
-        with audeer.progress_bar(
-                files,
-                total=len(files),
-                disable=not self.verbose,
-        ) as pbar:
-            for idx, file in enumerate(pbar):
-                desc = audeer.format_display_message(file, pbar=True)
-                pbar.set_description(desc, refresh=True)
-                mask = index.isin([file], 0)
-                select = index[mask].droplevel(0)
-                signal, sampling_rate = utils.read_audio(file)
-                ys[idx] = pd.Series(
-                    self.process_signal_from_index(
-                        signal, sampling_rate, select,
-                    ).values,
-                    index=index[mask],
-                )
-
-        return pd.concat(ys)
-
-    def __call__(
-        self,
-        signal: np.ndarray,
-        sampling_rate: int,
-        starts: typing.Sequence[int],
-        ends: typing.Sequence[int],
     ) -> typing.Any:
         r"""Apply processing to signal.
 
@@ -826,18 +855,4 @@ class ProcessWithContext:
             RuntimeError: if channel selection is invalid
 
         """
-        signal, sampling_rate = utils.preprocess_signal(
-            signal,
-            sampling_rate,
-            self.sampling_rate,
-            self.resample,
-            self.channels,
-            self.mixdown,
-        )
-        return self.process_func(
-            signal,
-            sampling_rate,
-            starts,
-            ends,
-            **self.process_func_kwargs,
-        )
+        return self._call(signal, sampling_rate)
