@@ -1,5 +1,7 @@
+import errno
 import os
 import typing
+import warnings
 
 import numpy as np
 import pandas as pd
@@ -22,13 +24,7 @@ def create_process_func(
 
     if process_func is None:
         def process_func(signal, sr, **kwargs):
-            return pd.MultiIndex.from_arrays(
-                [
-                    pd.to_timedelta([]),
-                    pd.to_timedelta([]),
-                ],
-                names=['start', 'end'],
-            )
+            return utils.signal_index()
 
     if invert:
         def process_func_invert(signal, sr, **kwargs):
@@ -53,13 +49,7 @@ def invert_index(
 
     """
     if index.empty:
-        return pd.MultiIndex.from_arrays(
-            [
-                [pd.to_timedelta(0)],
-                [dur],
-            ],
-            names=['start', 'end'],
-        )
+        return utils.signal_index(0, dur)
 
     starts = index.get_level_values('start')
     ends = index.get_level_values('end')
@@ -71,13 +61,7 @@ def invert_index(
     if ends[-1] != dur:
         new_starts = new_starts.insert(len(new_starts), ends[-1])
         new_ends = new_ends.insert(len(new_ends), dur)
-    return pd.MultiIndex.from_arrays(
-        [
-            new_starts,
-            new_ends,
-        ],
-        names=['start', 'end'],
-    )
+    return utils.signal_index(new_starts, new_ends)
 
 
 def merge_index(
@@ -108,13 +92,7 @@ def merge_index(
     new_starts.append(new_start)
     new_ends.append(new_end)
 
-    return pd.MultiIndex.from_arrays(
-        [
-            new_starts,
-            new_ends,
-        ],
-        names=['start', 'end'],
-    )
+    return utils.signal_index(new_starts, new_ends)
 
 
 class Segment:
@@ -127,17 +105,43 @@ class Segment:
         process_func: segmentation function,
             which expects the two positional arguments ``signal``
             and ``sampling_rate``
-            and any number of additional keyword arguments.
+            and any number of additional keyword arguments
+            (see ``process_func_args``).
+            There are the following special arguments:
+            ``'idx'``, ``'file'``, ``'root'``.
+            If expected by the function,
+            but not specified in
+            ``process_func_args``,
+            they will be replaced with:
+            a running index,
+            the currently processed file,
+            the root folder.
             Must return a :class:`pandas.MultiIndex` with two levels
             named `start` and `end` that hold start and end
-            positions as :class:`pandas.Timedelta` objects.
+            positions as :class:`pandas.Timedelta` objects
+        process_func_args: (keyword) arguments passed on to the processing
+            function
         invert: Invert the segmentation
-        sampling_rate: sampling rate in Hz.
+        sampling_rate: sampling rate in Hz
             If ``None`` it will call ``process_func`` with the actual
-            sampling rate of the signal.
+            sampling rate of the signal
         resample: if ``True`` enforces given sampling rate by resampling
         channels: channel selection, see :func:`audresample.remix`
         mixdown: apply mono mix-down on selection
+        min_signal_dur: minimum signal length
+            required by ``process_func``.
+            If value is a float or integer
+            it is treated as seconds.
+            See :func:`audinterface.utils.to_timedelta` for further options.
+            If provided signal is shorter,
+            it will be zero padded at the end
+        max_signal_dur: maximum signal length
+            required by ``process_func``.
+            If value is a float or integer
+            it is treated as seconds.
+            See :func:`audinterface.utils.to_timedelta` for further options.
+            If provided signal is longer,
+            it will be cut at the end
         keep_nat: if the end of segment is set to ``NaT`` do not replace
             with file duration in the result
         num_workers: number of parallel jobs or 1 for sequential
@@ -147,46 +151,101 @@ class Segment:
             multiprocessing
         multiprocessing: use multiprocessing instead of multithreading
         verbose: show debug messages
-        kwargs: additional keyword arguments to the processing function
 
     Raises:
         ValueError: if ``resample = True``, but ``sampling_rate = None``
 
-    """
+    Example:
+        >>> def segment(signal, sampling_rate, *, win_size=0.2, hop_size=0.1):
+        ...     size = signal.shape[1] / sampling_rate
+        ...     starts = pd.to_timedelta(np.arange(0, size - win_size, hop_size), unit='s')
+        ...     ends = pd.to_timedelta(np.arange(win_size, size, hop_size), unit='s')
+        ...     return pd.MultiIndex.from_tuples(zip(starts, ends), names=['start', 'end'])
+        >>> interface = Segment(process_func=segment)
+        >>> signal = np.array([1., 2., 3.])
+        >>> interface(signal, sampling_rate=3)
+        MultiIndex([(       '0 days 00:00:00', '0 days 00:00:00.200000'),
+                    ('0 days 00:00:00.100000', '0 days 00:00:00.300000'),
+                    ('0 days 00:00:00.200000', '0 days 00:00:00.400000'),
+                    ('0 days 00:00:00.300000', '0 days 00:00:00.500000'),
+                    ('0 days 00:00:00.400000', '0 days 00:00:00.600000'),
+                    ('0 days 00:00:00.500000', '0 days 00:00:00.700000'),
+                    ('0 days 00:00:00.600000', '0 days 00:00:00.800000'),
+                    ('0 days 00:00:00.700000', '0 days 00:00:00.900000')],
+                   names=['start', 'end'])
+        >>> import audb
+        >>> db = audb.load(
+        ...     'emodb',
+        ...     version='1.2.0',
+        ...     media='wav/03a01Fa.wav',
+        ...     full_path=False,
+        ...     verbose=False,
+        ... )
+        >>> interface = Segment(
+        ...     process_func=segment,
+        ...     process_func_args={'win_size': 0.5, 'hop_size': 0.25},
+        ... )
+        >>> interface.process_index(db['emotion'].index, root=db.root)
+        MultiIndex([('wav/03a01Fa.wav',        '0 days 00:00:00', ...),
+                    ('wav/03a01Fa.wav', '0 days 00:00:00.250000', ...),
+                    ('wav/03a01Fa.wav', '0 days 00:00:00.500000', ...),
+                    ('wav/03a01Fa.wav', '0 days 00:00:00.750000', ...),
+                    ('wav/03a01Fa.wav',        '0 days 00:00:01', ...),
+                    ('wav/03a01Fa.wav', '0 days 00:00:01.250000', ...)],
+                   names=['file', 'start', 'end'])
+
+    """  # noqa: E501
     def __init__(
             self,
             *,
             process_func: typing.Callable[..., pd.MultiIndex] = None,
+            process_func_args: typing.Dict[str, typing.Any] = None,
             invert: bool = False,
             sampling_rate: int = None,
             resample: bool = False,
             channels: typing.Union[int, typing.Sequence[int]] = None,
             mixdown: bool = False,
+            min_signal_dur: Timestamp = None,
+            max_signal_dur: Timestamp = None,
             keep_nat: bool = False,
             num_workers: typing.Optional[int] = 1,
             multiprocessing: bool = False,
             verbose: bool = False,
             **kwargs,
     ):
-        self.invert = invert
-        r"""Invert segmentation."""
+        process_func_args = process_func_args or {}
+        if kwargs:
+            warnings.warn(
+                utils.kwargs_deprecation_warning,
+                category=UserWarning,
+                stacklevel=2,
+            )
+            for key, value in kwargs.items():
+                process_func_args[key] = value
 
         # avoid cycling imports
         from audinterface.core.process import Process
-        self.process = Process(
+        process = Process(
             process_func=create_process_func(process_func, invert),
+            process_func_args=process_func_args,
             sampling_rate=sampling_rate,
             resample=resample,
             channels=channels,
             mixdown=mixdown,
+            min_signal_dur=min_signal_dur,
+            max_signal_dur=max_signal_dur,
             keep_nat=keep_nat,
             num_workers=num_workers,
             multiprocessing=multiprocessing,
             verbose=verbose,
-            **kwargs,
         )
+
+        self.process = process
         r"""Processing object."""
 
+        self.invert = invert
+        r"""Invert segmentation."""
+
     def process_file(
             self,
             file: str,
@@ -200,9 +259,11 @@ class Segment:
         Args:
             file: file path
             start: start processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
             end: end processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
             root: root folder to expand relative file path
 
         Returns:
@@ -223,18 +284,10 @@ class Segment:
             end=end,
             root=root,
         ).values[0]
-        return pd.MultiIndex(
-            levels=[
-                [file],
-                index.levels[0] + start,
-                index.levels[1] + start,
-            ],
-            codes=[
-                [0] * len(index),
-                index.codes[0],
-                index.codes[1],
-            ],
-            names=['file', 'start', 'end'],
+        return audformat.segmented_index(
+            files=[file] * len(index),
+            starts=index.levels[0] + start,
+            ends=index.levels[1] + start,
         )
 
     def process_files(
@@ -251,9 +304,13 @@ class Segment:
             files: list of file paths
             starts: segment start positions.
                 Time values given as float or integers are treated as seconds.
+                See :func:`audinterface.utils.to_timedelta`
+                for further options.
                 If a scalar is given, it is applied to all files
             ends: segment end positions.
                 Time values given as float or integers are treated as seconds
+                See :func:`audinterface.utils.to_timedelta`
+                for further options.
                 If a scalar is given, it is applied to all files
             root: root folder to expand relative file paths
 
@@ -273,21 +330,15 @@ class Segment:
             ends=ends,
             root=root,
         )
+        if len(series) == 0:
+            return audformat.filewise_index()
         objs = []
         for idx, ((file, start, _), index) in enumerate(series.items()):
             objs.append(
-                pd.MultiIndex(
-                    levels=[
-                        [file],
-                        index.levels[0] + start,
-                        index.levels[1] + start,
-                    ],
-                    codes=[
-                        [0] * len(index),
-                        index.codes[0],
-                        index.codes[1],
-                    ],
-                    names=['file', 'start', 'end'],
+                audformat.segmented_index(
+                    files=[file] * len(index),
+                    starts=index.levels[0] + start,
+                    ends=index.levels[1] + start,
                 )
             )
         return audformat.utils.union(objs)
@@ -310,12 +361,21 @@ class Segment:
             Segmented index conform to audformat_
 
         Raises:
+            FileNotFoundError: if folder does not exist
             RuntimeError: if sampling rates do not match
             RuntimeError: if channel selection is invalid
 
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
+        root = audeer.safe_path(root)
+        if not os.path.exists(root):
+            raise FileNotFoundError(
+                errno.ENOENT,
+                os.strerror(errno.ENOENT),
+                root,
+            )
+
         files = audeer.list_file_names(root, filetype=filetype)
         files = [os.path.join(root, os.path.basename(f)) for f in files]
         return self.process_files(files)
@@ -343,7 +403,7 @@ class Segment:
 
         """
         index = audformat.utils.to_segmented_index(index)
-        utils.check_index(index)
+        utils.assert_index(index)
 
         if index.empty:
             return index
@@ -375,9 +435,11 @@ class Segment:
             sampling_rate: sampling rate in Hz
             file: file path
             start: start processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
             end: end processing at this position.
-                If value is as a float or integer it is treated as seconds
+                If value is a float or integer it is treated as seconds.
+                See :func:`audinterface.utils.to_timedelta` for further options
 
         Returns:
             Segmented index conform to audformat_
@@ -396,26 +458,20 @@ class Segment:
             start=start,
             end=end,
         ).values[0]
-        utils.check_index(index)
+        utils.assert_index(index)
         if start is not None:
             index = index.set_levels(
                 [
                     index.levels[0] + start,
                     index.levels[1] + start,
-                ], [0, 1])
+                ],
+                level=[0, 1],
+            )
         if file is not None:
-            index = pd.MultiIndex(
-                levels=[
-                    [file],
-                    index.levels[0],
-                    index.levels[1],
-                ],
-                codes=[
-                    [0] * len(index),
-                    index.codes[0],
-                    index.codes[1],
-                ],
-                names=['file', 'start', 'end'],
+            index = audformat.segmented_index(
+                files=[file] * len(index),
+                starts=index.levels[0],
+                ends=index.levels[1],
             )
         return index
 
@@ -434,6 +490,7 @@ class Segment:
                 or a :class:`pandas.MultiIndex` with two levels
                 named `start` and `end` that hold start and end
                 positions as :class:`pandas.Timedelta` objects.
+                See also :func:`audinterface.utils.signal_index`
 
         Returns:
             Segmented index conform to audformat_
@@ -441,11 +498,12 @@ class Segment:
         Raises:
             RuntimeError: if sampling rates do not match
             RuntimeError: if channel selection is invalid
+            ValueError: if index contains duplicates
 
         .. _audformat: https://audeering.github.io/audformat/data-format.html
 
         """
-        utils.check_index(index)
+        utils.assert_index(index)
 
         if index.empty:
             return index
@@ -474,10 +532,7 @@ class Segment:
             progress_bar=self.process.verbose,
             task_description=f'Process {len(index)} segments',
         )
-
-        index = y[0]
-        for obj in y[1:]:
-            index = index.union(obj)
+        index = audformat.utils.union(y)
 
         return index
 
