@@ -4,6 +4,7 @@ import numpy as np
 import pandas as pd
 import pytest
 
+import audeer
 import audformat
 import audinterface
 import audiofile as af
@@ -15,24 +16,11 @@ NUM_FEATURES = 3
 NUM_FRAMES = 5
 SIGNAL_1D = np.ones((1, SAMPLING_RATE))
 SIGNAL_2D = np.ones((NUM_CHANNELS, SAMPLING_RATE))
-STARTS = [pd.to_timedelta('0s')] * 3
-ENDS = [pd.to_timedelta('1s')] * 3
-INDEX = pd.MultiIndex.from_arrays(
-    [STARTS, ENDS],
-    names=['start', 'end']
-)
 SEGMENT = audinterface.Segment(
     process_func=lambda x, sr:
-        pd.MultiIndex.from_arrays(
-            [
-                [
-                    pd.to_timedelta(0),
-                ],
-                [
-                    pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
-                ],
-            ],
-            names=['start', 'end'],
+        audinterface.utils.signal_index(
+            pd.to_timedelta(0),
+            pd.to_timedelta(x.shape[1] / sr, unit='s') / 2,
         )
 )
 
@@ -41,24 +29,89 @@ def feature_extractor(signal, _):
     return np.ones((NUM_CHANNELS, NUM_FEATURES))
 
 
-def features_extractor_sliding_window(signal, _, hop_size):
-    num_time_steps = int(np.ceil(signal.shape[1] / hop_size))
-    return np.ones((NUM_CHANNELS, NUM_FEATURES, num_time_steps))
+def mean(signal, sampling_rate):
+    return signal.mean(axis=1, keepdims=True)
+
+
+def mean_mono(signal, sampling_rate):
+    return signal.mean()
+
+
+def mean_sliding_window(signal, sampling_rate, win_dur, hop_dur):
+    frames = audinterface.utils.sliding_window(
+        signal,
+        sampling_rate,
+        win_dur,
+        hop_dur,
+    )
+    return frames.mean(axis=1, keepdims=True)
+
+
+def mean_sliding_window_mono(signal, sampling_rate, win_dur, hop_dur):
+    frames = audinterface.utils.sliding_window(
+        signal,
+        sampling_rate,
+        win_dur,
+        hop_dur,
+    )
+    return frames.mean(axis=1, keepdims=False)
+
+
+def test_deprecated_process_func_applies_sliding_window_argument():
+    interface = audinterface.Feature(
+        'mean',
+        process_func=mean,
+    )
+    if (
+            audeer.LooseVersion(audinterface.__version__)
+            < audeer.LooseVersion('1.2.0')
+    ):
+        assert interface.process_func_applies_sliding_window
+    else:
+        assert not interface.process_func_applies_sliding_window
+
+
+def test_deprecated_unit_argument():
+    if (
+            audeer.LooseVersion(audinterface.__version__)
+            < audeer.LooseVersion('1.2.0')
+    ):
+        with pytest.warns(UserWarning, match='is deprecated'):
+            interface = audinterface.Feature(
+                ['a'],
+                win_dur=1000,
+                unit='samples',
+                sampling_rate=16000,
+            )
+            assert interface.win_dur == '1000'
+            interface = audinterface.Feature(
+                ['a'],
+                win_dur=1000,
+                hop_dur=500,
+                unit='milliseconds',
+            )
+            assert interface.win_dur == '1000milliseconds'
+            assert interface.hop_dur == '500milliseconds'
+    else:
+        with pytest.raises(TypeError, match='unexpected keyword argument'):
+            audinterface.Feature(
+                ['a'],
+                unit='samples',
+            )
 
 
 def test_feature():
-    # You have to specify sampling rate with unit == 'samples' and win_dur
+    # You have to specify sampling rate when win_dur is in samples
     with pytest.raises(ValueError):
         audinterface.Feature(
             feature_names=('o1', 'o2', 'o3'),
+            process_func_applies_sliding_window=False,
             sampling_rate=None,
-            unit='samples',
-            win_dur=2048,
+            win_dur='2048',
         )
     # If no win_dur is given, no error should occur
     audinterface.Feature(
         feature_names=('o1', 'o2', 'o3'),
-        unit='samples',
         sampling_rate=None,
     )
     # Only hop_dur is given
@@ -69,8 +122,8 @@ def test_feature():
         )
     audinterface.Feature(
         feature_names=('o1', 'o2', 'o3'),
-        win_dur=2048,
-        unit='samples',
+        process_func_applies_sliding_window=False,
+        win_dur='2048',
         sampling_rate=8000,
     )
 
@@ -78,13 +131,45 @@ def test_feature():
 @pytest.mark.parametrize(
     'signal, feature, expected',
     [
+        (
+            SIGNAL_1D,
+            audinterface.Feature(
+                feature_names='feature',
+                process_func=lambda x, sr: 1,
+            ),
+            np.ones((1, 1, 1)),
+        ),
+        (
+            SIGNAL_1D,
+            audinterface.Feature(
+                feature_names=['feature'],
+                process_func=lambda x, sr: 1,
+            ),
+            np.ones((1, 1, 1)),
+        ),
+        (
+            SIGNAL_1D,
+            audinterface.Feature(
+                feature_names=['f1', 'f2', 'f3'],
+                process_func=lambda x, sr: np.ones(3),
+            ),
+            np.ones((1, 3, 1)),
+        ),
         (
             SIGNAL_1D,
             audinterface.Feature(
                 feature_names=['f1', 'f2', 'f3'],
                 process_func=lambda x, sr: np.ones((1, 3)),
             ),
-            np.ones((1, 3)),
+            np.ones((1, 3, 1)),
+        ),
+        (
+            SIGNAL_1D,
+            audinterface.Feature(
+                feature_names=['f1', 'f2', 'f3'],
+                process_func=lambda x, sr: np.ones((3, 1)),
+            ),
+            np.ones((1, 3, 1)),
         ),
         (
             SIGNAL_1D,
@@ -110,7 +195,7 @@ def test_feature():
                 process_func=lambda x, sr: np.ones((2, 3)),
                 channels=range(2),
             ),
-            np.ones((2, 3)),
+            np.ones((2, 3, 1)),
         ),
         (
             SIGNAL_2D,
@@ -121,6 +206,16 @@ def test_feature():
             ),
             np.ones((2, 3, 5)),
         ),
+        (
+            SIGNAL_2D,
+            audinterface.Feature(
+                feature_names=['f1', 'f2', 'f3'],
+                process_func=lambda x, sr: np.ones(3),
+                channels=range(2),
+                process_func_is_mono=True,
+            ),
+            np.ones((2, 3, 1)),
+        ),
         (
             SIGNAL_2D,
             audinterface.Feature(
@@ -129,7 +224,27 @@ def test_feature():
                 channels=range(2),
                 process_func_is_mono=True,
             ),
-            np.ones((2, 3)),
+            np.ones((2, 3, 1)),
+        ),
+        (
+            SIGNAL_2D,
+            audinterface.Feature(
+                feature_names=['f1', 'f2', 'f3'],
+                process_func=lambda x, sr: np.ones((1, 3, 1)),
+                channels=range(2),
+                process_func_is_mono=True,
+            ),
+            np.ones((2, 3, 1)),
+        ),
+        (
+            SIGNAL_2D,
+            audinterface.Feature(
+                feature_names=['f1', 'f2', 'f3'],
+                process_func=lambda x, sr: np.ones((3, 5)),
+                channels=range(2),
+                process_func_is_mono=True,
+            ),
+            np.ones((2, 3, 5)),
         ),
         (
             SIGNAL_2D,
@@ -143,13 +258,45 @@ def test_feature():
         ),
     ]
 )
-def test_process_callable(signal, feature, expected):
+def test_process_call(signal, feature, expected):
     np.testing.assert_array_equal(
         feature(signal, SAMPLING_RATE),
         expected,
     )
 
 
+@pytest.mark.parametrize(
+    'signal, extractor',
+    [
+        (
+            np.random.randn(1, SAMPLING_RATE),
+            audinterface.Feature(
+                feature_names='mean',
+                process_func=lambda x, sr: np.mean(x, axis=1),
+            ),
+        ),
+        (
+            np.random.randn(2, SAMPLING_RATE),
+            audinterface.Feature(
+                feature_names='mean',
+                process_func=lambda x, sr: np.mean(x, axis=1),
+            ),
+        ),
+        (
+            np.random.randn(2, SAMPLING_RATE),
+            audinterface.Feature(
+                feature_names='mean',
+                process_func=lambda x, sr: np.mean(x, axis=1),
+                channels=0,
+            ),
+        ),
+    ]
+)
+def test_process_call_datatype(signal, extractor):
+    features = extractor(signal, SAMPLING_RATE)
+    assert signal.dtype == features.dtype
+
+
 @pytest.mark.parametrize(
     'start, end, segment',
     [
@@ -227,34 +374,78 @@ def test_process_file(tmpdir, start, end, segment):
 
 
 def test_process_folder(tmpdir):
-    extractor = audinterface.Feature(
-        feature_names=('o1', 'o2', 'o3'),
+
+    index = audinterface.utils.signal_index(0, 1)
+    feature_names = ['o1', 'o2', 'o3']
+    feature = audinterface.Feature(
+        feature_names,
         process_func=feature_extractor,
         sampling_rate=None,
         channels=range(NUM_CHANNELS),
         resample=False,
         verbose=False,
     )
-    expected_features = np.ones((3, NUM_CHANNELS * NUM_FEATURES))
+
     path = str(tmpdir.mkdir('wav'))
     files = [
         os.path.join(path, f'file{n}.wav') for n in range(3)
     ]
     for file in files:
         af.write(file, SIGNAL_2D, SAMPLING_RATE)
-    features = extractor.process_folder(path)
-    assert all(features.index.levels[0] == files)
-    assert all(features.index.levels[1] == INDEX.levels[0])
-    assert all(features.index.levels[2] == INDEX.levels[1])
-    np.testing.assert_array_equal(features.values, expected_features)
+
+    y = feature.process_folder(path)
+    y_expected = np.ones((3, NUM_CHANNELS * NUM_FEATURES))
+
+    assert all(y.index.levels[0] == files)
+    assert all(y.index.levels[1] == index.levels[0])
+    assert all(y.index.levels[2] == index.levels[1])
+    np.testing.assert_array_equal(y.values, y_expected)
+
+    # non-existing folder
+    with pytest.raises(FileNotFoundError):
+        feature.process_folder('bad-folder')
+
+    # empty folder
+    root = str(tmpdir.mkdir('empty'))
+    df = feature.process_folder(root)
+    pd.testing.assert_frame_equal(
+        df,
+        pd.DataFrame(
+            dtype=object,
+            columns=feature.column_names,
+        ),
+    )
+
+
+def test_process_func_args():
+    def process_func(s, sr, arg1, arg2):
+        assert arg1 == 'foo'
+        assert arg2 == 'bar'
+    audinterface.Feature(
+        feature_names=('o1', 'o2', 'o3'),
+        process_func=process_func,
+        process_func_args={
+            'arg1': 'foo',
+            'arg2': 'bar',
+        }
+    )
+    with pytest.warns(UserWarning):
+        audinterface.Feature(
+            feature_names=('o1', 'o2', 'o3'),
+            process_func=process_func,
+            arg1='foo',
+            arg2='bar',
+        )
 
 
 @pytest.mark.parametrize(
-    'process_func, num_feat, signal, start, end, expand, expected',
+    'process_func, applies_sliding_window, num_feat, signal, start, end, '
+    'expand, expected',
     [
         # no process function
         (
             None,
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -263,8 +454,40 @@ def test_process_folder(tmpdir):
             np.zeros((1, 2 * 3)),
         ),
         # 1 channel, 1 feature
+        (
+            lambda s, sr: 1,
+            False,
+            1,
+            SIGNAL_1D,
+            None,
+            None,
+            False,
+            np.ones((1, 1)),
+        ),
+        (
+            lambda s, sr: np.ones(1),
+            False,
+            1,
+            SIGNAL_1D,
+            None,
+            None,
+            False,
+            np.ones((1, 1)),
+        ),
+        # 1 channel, 1 feature
+        (
+            lambda s, sr: [1],
+            False,
+            1,
+            SIGNAL_1D,
+            None,
+            None,
+            False,
+            np.ones((1, 1)),
+        ),
         (
             lambda s, sr: np.ones((1, 1)),
+            False,
             1,
             SIGNAL_1D,
             None,
@@ -273,8 +496,40 @@ def test_process_folder(tmpdir):
             np.ones((1, 1)),
         ),
         # 1 channel, 3 features
+        (
+            lambda s, sr: [1, 1, 1],
+            False,
+            3,
+            SIGNAL_1D,
+            None,
+            None,
+            False,
+            np.ones((1, 3)),
+        ),
+        (
+            lambda s, sr: np.ones(3),
+            False,
+            3,
+            SIGNAL_1D,
+            None,
+            None,
+            False,
+            np.ones((1, 3)),
+        ),
+        # 1 channel, 3 features
+        (
+            lambda s, sr: [[1, 1, 1]],
+            False,
+            3,
+            SIGNAL_1D,
+            None,
+            None,
+            False,
+            np.ones((1, 3)),
+        ),
         (
             lambda s, sr: np.ones((1, 3)),
+            False,
             3,
             SIGNAL_1D,
             None,
@@ -283,8 +538,19 @@ def test_process_folder(tmpdir):
             np.ones((1, 3)),
         ),
         # 2 channels, 1 feature
+        (
+            lambda s, sr: [[1], [1]],
+            False,
+            1,
+            SIGNAL_2D,
+            None,
+            None,
+            False,
+            np.ones((1, 2)),
+        ),
         (
             lambda s, sr: np.ones((2, 1)),
+            False,
             1,
             SIGNAL_2D,
             None,
@@ -293,8 +559,19 @@ def test_process_folder(tmpdir):
             np.ones((1, 2)),
         ),
         # 2 channels, 3 features
+        (
+            lambda s, sr: [[1, 1, 1], [1, 1, 1]],
+            False,
+            3,
+            SIGNAL_2D,
+            None,
+            None,
+            False,
+            np.ones((1, 2 * 3)),
+        ),
         (
             lambda s, sr: np.ones((2, 3)),
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -305,6 +582,7 @@ def test_process_folder(tmpdir):
         # 2 channels, 3 features + start, end
         (
             lambda s, sr: np.ones((2, 3)),
+            True,
             3,
             SIGNAL_2D,
             pd.to_timedelta('1s'),
@@ -312,9 +590,10 @@ def test_process_folder(tmpdir):
             False,
             np.ones((1, 2 * 3)),
         ),
-        # 2 channels, 3 features, 5 steps
+        # 2 channels, 3 features, 5 frames
         (
-            lambda s, sr: np.ones((2, 3, 5)),
+            lambda s, sr: [[[1] * 5] * 3] * 2,
+            True,
             3,
             SIGNAL_2D,
             None,
@@ -322,9 +601,30 @@ def test_process_folder(tmpdir):
             False,
             np.ones((5, 2 * 3)),
         ),
-        # 1 channel, 1 feature + expand
         (
-            lambda s, sr: np.ones((1, 1)),
+            lambda s, sr: np.ones((2, 3, 5)),
+            True,
+            3,
+            SIGNAL_2D,
+            None,
+            None,
+            False,
+            np.ones((5, 2 * 3)),
+        ),
+        # 1 channel, 1 feature + mono processing
+        (
+            lambda s, sr: 1,
+            False,
+            1,
+            SIGNAL_1D,
+            None,
+            None,
+            True,
+            np.ones((1, 1)),
+        ),
+        (
+            lambda s, sr: np.ones(1),
+            False,
             1,
             SIGNAL_1D,
             None,
@@ -332,9 +632,20 @@ def test_process_folder(tmpdir):
             True,
             np.ones((1, 1)),
         ),
-        # 2 channels, 1 feature + expand
         (
             lambda s, sr: np.ones((1, 1)),
+            False,
+            1,
+            SIGNAL_1D,
+            None,
+            None,
+            True,
+            np.ones((1, 1)),
+        ),
+        # 2 channels, 1 feature + mono processing
+        (
+            lambda s, sr: [1],
+            False,
             1,
             SIGNAL_2D,
             None,
@@ -342,9 +653,30 @@ def test_process_folder(tmpdir):
             True,
             np.ones((1, 2)),
         ),
-        # 2 channels, 3 features + expand
         (
-            lambda s, sr: np.ones((1, 3)),
+            lambda s, sr: np.ones(1),
+            False,
+            1,
+            SIGNAL_2D,
+            None,
+            None,
+            True,
+            np.ones((1, 2)),
+        ),
+        (
+            lambda s, sr: np.ones((1, 1)),
+            False,
+            1,
+            SIGNAL_2D,
+            None,
+            None,
+            True,
+            np.ones((1, 2)),
+        ),
+        # 2 channels, 3 features + mono processing
+        (
+            lambda s, sr: [1, 1, 1],
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -352,9 +684,34 @@ def test_process_folder(tmpdir):
             True,
             np.ones((1, 2 * 3)),
         ),
-        # 2 channels, 3 features, 5 steps + expand
         (
-            lambda s, sr: np.ones((1, 3, 5)),
+            lambda s, sr: np.ones(3),
+            False,
+            3,
+            SIGNAL_2D,
+            None,
+            None,
+            True,
+            np.ones((1, 2 * 3)),
+        ),
+        (
+            lambda s, sr: np.ones((1, 3, 1)),
+            False,
+            3,
+            SIGNAL_2D,
+            None,
+            None,
+            True,
+            np.ones((1, 2 * 3)),
+        ),
+        # 2 channels, 3 features, 5 frames + mono processing
+        (
+            lambda s, sr: [
+                [1, 1, 1, 1, 1],
+                [1, 1, 1, 1, 1],
+                [1, 1, 1, 1, 1],
+            ],
+            True,
             3,
             SIGNAL_2D,
             None,
@@ -362,20 +719,30 @@ def test_process_folder(tmpdir):
             True,
             np.ones((5, 2 * 3)),
         ),
-        # Feature extractor function is not a np.ndarray
-        pytest.param(
-            lambda s, sr: 1,
+        (
+            lambda s, sr: np.ones((3, 5)),
+            True,
             3,
             SIGNAL_2D,
             None,
             None,
+            True,
+            np.ones((5, 2 * 3)),
+        ),
+        (
+            lambda s, sr: np.ones((1, 3, 5)),
+            True,
+            3,
+            SIGNAL_2D,
             None,
-            False,
-            marks=pytest.mark.xfail(raises=RuntimeError),
+            None,
+            True,
+            np.ones((5, 2 * 3)),
         ),
         # Feature extractor function returns too less dimensions
         pytest.param(
-            lambda s, sr: np.ones((1, )),
+            lambda s, sr: np.ones(1),
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -387,6 +754,7 @@ def test_process_folder(tmpdir):
         # Feature extractor function returns too many dimensions
         pytest.param(
             lambda s, sr: np.ones((1, 1, 1, 1)),
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -398,6 +766,7 @@ def test_process_folder(tmpdir):
         # Feature extractor function returns wrong number of channels
         pytest.param(
             lambda s, sr: np.ones((1, 3)),
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -409,6 +778,7 @@ def test_process_folder(tmpdir):
         # Feature extractor function returns wrong number of channels
         pytest.param(
             lambda s, sr: np.ones((2 + 1, 3)),
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -420,6 +790,19 @@ def test_process_folder(tmpdir):
         # Feature extractor function returns wrong number of features
         pytest.param(
             lambda s, sr: np.ones((2, 3 + 1)),
+            False,
+            3,
+            SIGNAL_2D,
+            None,
+            None,
+            None,
+            False,
+            marks=pytest.mark.xfail(raises=RuntimeError),
+        ),
+        # Feature extractor function returns wrong number of features
+        pytest.param(
+            lambda s, sr: np.ones((2, 3 + 1, 1)),
+            False,
             3,
             SIGNAL_2D,
             None,
@@ -431,33 +814,33 @@ def test_process_folder(tmpdir):
     ]
 )
 def test_process_signal(
-        process_func, num_feat, signal, start, end, expand, expected,
+        process_func, applies_sliding_window, num_feat, signal, start, end,
+        expand, expected,
 ):
-    extractor = audinterface.Feature(
+    feature = audinterface.Feature(
         feature_names=[f'f{i}' for i in range(num_feat)],
         process_func=process_func,
+        process_func_applies_sliding_window=applies_sliding_window,
         channels=range(signal.shape[0]),
         process_func_is_mono=expand,
+        win_dur=1,
     )
-    features = extractor.process_signal(
+    y = feature.process_signal(
         signal,
         SAMPLING_RATE,
         start=start,
         end=end,
     )
-    np.testing.assert_array_equal(features.values, expected)
+    np.testing.assert_array_equal(y.values, expected)
 
 
 @pytest.mark.parametrize(
     'index,expected_features',
     [
         (
-            pd.MultiIndex.from_arrays(
-                [
-                    (pd.to_timedelta('0s'), pd.to_timedelta('1s')),
-                    (pd.to_timedelta('2s'), pd.to_timedelta('3s')),
-                ],
-                names=['start', 'end'],
+            audinterface.utils.signal_index(
+                [pd.to_timedelta('0s'), pd.to_timedelta('1s')],
+                [pd.to_timedelta('2s'), pd.to_timedelta('3s')],
             ),
             np.ones((2, NUM_CHANNELS * NUM_FEATURES)),
         ),
@@ -479,6 +862,8 @@ def test_process_signal_from_index(index, expected_features):
 
 def test_process_index(tmpdir):
 
+    cache_root = os.path.join(tmpdir, 'cache')
+
     feature = audinterface.Feature(
         feature_names=('o1', 'o2', 'o3'),
         process_func=feature_extractor,
@@ -488,108 +873,149 @@ def test_process_index(tmpdir):
     # empty
 
     index = audformat.segmented_index()
-    y = feature.process_index(index)
-    assert y.empty
-    assert y.columns.tolist() == feature.column_names
+    df = feature.process_index(index)
+    assert df.empty
+    assert df.columns.tolist() == feature.column_names
 
     # non-empty
 
     # create file
     root = str(tmpdir.mkdir('wav'))
-    file = 'file.wav'
-    path = os.path.join(root, file)
-    af.write(path, SIGNAL_2D, SAMPLING_RATE)
-    y_expected = np.ones((2, NUM_CHANNELS * NUM_FEATURES))
+    files = ['file-1.wav', 'file-2.wav']
+    paths = [os.path.join(root, file) for file in files]
+    for path in paths:
+        af.write(path, SIGNAL_2D, SAMPLING_RATE)
+    df_expected = np.ones((2, NUM_CHANNELS * NUM_FEATURES))
 
     # absolute paths
-    index = audformat.segmented_index([path] * 2, [0, 1], [2, 3])
-    y = feature.process_index(index)
-    assert y.index.get_level_values('file')[0] == path
-    np.testing.assert_array_equal(y.values, y_expected)
-    assert y.columns.tolist() == feature.column_names
+    index = audformat.segmented_index(paths, [0, 1], [2, 3])
+    df = feature.process_index(index)
+    assert df.index.get_level_values('file')[0] == paths[0]
+    np.testing.assert_array_equal(df.values, df_expected)
+    assert df.columns.tolist() == feature.column_names
+    df = feature.process_index(index)
 
     # relative paths
-    index = audformat.segmented_index([file] * 2, [0, 1], [2, 3])
-    y = feature.process_index(index, root=root)
-    assert y.index.get_level_values('file')[0] == file
-    np.testing.assert_array_equal(y.values, y_expected)
-    assert y.columns.tolist() == feature.column_names
+    index = audformat.segmented_index(files, [0, 1], [2, 3])
+    df = feature.process_index(index, root=root)
+    assert df.index.get_level_values('file')[0] == files[0]
+    np.testing.assert_array_equal(df.values, df_expected)
+    assert df.columns.tolist() == feature.column_names
+
+    # cache result
+    df = feature.process_index(
+        index,
+        root=root,
+        cache_root=cache_root,
+    )
+    os.remove(paths[1])
+
+    # fails because second file does not exist
+    with pytest.raises(RuntimeError):
+        feature.process_index(
+            index,
+            root=root,
+        )
+
+    # loading from cache still works
+    df_cached = feature.process_index(
+        index,
+        root=root,
+        cache_root=cache_root,
+    )
+    pd.testing.assert_frame_equal(df, df_cached)
 
 
 @pytest.mark.parametrize(
-    'win_dur, hop_dur, unit',
+    'process_func, is_mono, applies_sliding_window, feature_names',
     [
-        (1, 0.5, 'seconds'),
-        (1, None, 'seconds'),
-        (1000, 500, 'milliseconds'),
-        (None, None, 'seconds'),
-        (SAMPLING_RATE, SAMPLING_RATE // 2, 'samples'),
-        (None, None, 'samples'),
+        (mean, False, False, 'mean'),
+        (mean_mono, True, False, 'mean'),
+        (mean_sliding_window, False, True, 'mean'),
+        (mean_sliding_window_mono, True, True, 'mean'),
+    ]
+)
+@pytest.mark.parametrize(
+    'win_dur, hop_dur',
+    [
+        (0.5, None),
+        (0.5, 0.25),
+        (0.5, 0.5),
+        (0.25, 0.5),
+        (pd.to_timedelta(1, unit='s'), pd.to_timedelta(0.5, unit='s')),
+        ('4000', '2000'),
+        ('500ms', '250ms'),
+        ('500milliseconds', '250milliseconds'),
+        (f'{SAMPLING_RATE // 2}', f'{SAMPLING_RATE // 4}'),
     ],
 )
-def test_signal_sliding_window(win_dur, hop_dur, unit):
-    # Test sliding window with two time steps
-    expected_features = np.ones((NUM_CHANNELS, 2 * NUM_FEATURES))
-    extractor = audinterface.Feature(
-        feature_names=('o1', 'o2', 'o3'),
-        process_func=features_extractor_sliding_window,
+def test_signal_sliding_window(process_func, is_mono, applies_sliding_window,
+                               feature_names, win_dur, hop_dur):
+
+    interface = audinterface.Feature(
+        feature_names=feature_names,
+        process_func=process_func,
+        process_func_is_mono=is_mono,
+        process_func_applies_sliding_window=applies_sliding_window,
         channels=range(NUM_CHANNELS),
         win_dur=win_dur,
         hop_dur=hop_dur,
         sampling_rate=SAMPLING_RATE,
-        unit=unit,
-        hop_size=SAMPLING_RATE // 2,  # argument to process_func
     )
-    features = extractor.process_signal(
-        SIGNAL_2D,
-        SAMPLING_RATE,
-    )
-    n_time_steps = len(features)
-    start = pd.to_timedelta(0)
-    end = pd.to_timedelta(SIGNAL_2D.shape[-1] / SAMPLING_RATE, unit='seconds')
-    if win_dur is None:
-        starts = [start] * n_time_steps
-        ends = [end] * n_time_steps
-    else:
-        if unit == 'samples':
-            win_dur = win_dur / SAMPLING_RATE
-            if hop_dur is not None:
-                hop_dur /= SAMPLING_RATE
-            unit = 'seconds'
+
+    for signal in [SIGNAL_1D, SIGNAL_2D]:
+
+        df = interface.process_signal(
+            SIGNAL_2D,
+            SAMPLING_RATE,
+        )
+        n_time_steps = len(df)
+
+        win_dur = audinterface.utils.to_timedelta(win_dur, SAMPLING_RATE)
         if hop_dur is None:
             hop_dur = win_dur / 2
+        hop_dur = audinterface.utils.to_timedelta(hop_dur, SAMPLING_RATE)
 
         starts = pd.timedelta_range(
-            start,
-            freq=pd.to_timedelta(hop_dur, unit=unit),
+            pd.to_timedelta(0),
+            freq=hop_dur,
             periods=n_time_steps,
         )
-        ends = starts + pd.to_timedelta(win_dur, unit=unit)
-    index = pd.MultiIndex.from_arrays(
-        [starts, ends],
-        names=['start', 'end'],
-    )
-    pd.testing.assert_frame_equal(
-        features,
-        pd.DataFrame(
-            expected_features,
+        ends = starts + win_dur
+
+        index = audinterface.utils.signal_index(starts, ends)
+        expected = pd.DataFrame(
+            np.ones((n_time_steps, len(interface.column_names))),
             index=index,
-            columns=extractor.column_names,
-        ),
-    )
+            columns=interface.column_names,
+        )
+        pd.testing.assert_frame_equal(df, expected)
 
 
-def test_signal_kwargs():
-    def process_func(s, sr, arg1, arg2):
-        assert arg1 == 'foo'
-        assert arg2 == 'bar'
-    audinterface.Feature(
-        feature_names=('o1', 'o2', 'o3'),
-        process_func=process_func,
-        arg1='foo',
-        arg2='bar',
+def test_signal_sliding_window_error():
+
+    interface = audinterface.Feature(
+        feature_names='mean',
+        process_func=mean_sliding_window,
+        process_func_args={
+            'win_dur': 0.5,
+            'hop_dur': 0.25,
+        },
+        process_func_is_mono=False,
+        process_func_applies_sliding_window=True,
+        channels=range(NUM_CHANNELS),
+        win_dur=None,
+        hop_dur=None,
+        sampling_rate=SAMPLING_RATE,
     )
 
+    # returns multiple frames but win_dur is None
+    with pytest.raises(RuntimeError):
+        interface.process_signal(
+            SIGNAL_2D,
+            SAMPLING_RATE,
+        )
+
 
 def test_to_numpy():
     expected_features = np.ones((NUM_CHANNELS, NUM_FEATURES, 1))
